%% disconnect everything
instrreset;

%% scan for connected LoRaWAN DemoNodes
a = udp('192.168.0.255', 2390);
fopen(a);

node = [];

for loop = 1:3
    fwrite(a, ['LoRa_RST!'], 'uint8');                                      % hardware-reset the node
    fwrite(a, ['LoRa_RCN!'], 'uint8');                                      % drop existing connections
    fwrite(a, ['LoRa_SCAN'], 'uint8');                                      % scan for nodes
    pause(0.3);
    while a.BytesAvailable
        msg = fread(a, a.BytesAvailable).';
        %try
            if isequal(char(msg(1:9)), 'DemoNode;')
                params  = strsplit(char(msg), ';');
                ipaddr  = params{2};
                version = params{3};
                if isempty(node) || ~any(strcmp({node.ipaddr}, ipaddr))
                    fprintf('DemoNode found with IP=%s, v=%s\n', ipaddr, version);
                    node(end+1).ipaddr  = ipaddr;
                    node(end).demonode_version = str2double(version);
                end
            end
        %catch
        %end
    end
end
fclose(a);

if isempty(node)
    error('no nodes found');
else
    fprintf('%d nodes found in total.\n', length(node));
    [~, idx] = sort({node.ipaddr});                                         % sort the result by IP address
    node = node(idx);
    
    node(1).device_name = [];
    node(1).rn2483_version = [];
    node(1).hweui_hex = [];
    node(1).devaddr = [];
end

%% connect nodes
for idx = 1:length(node)
    node(idx).handle = tcpip(node(idx).ipaddr, 2390);
    node(idx).handle.Terminator = 'CR/LF';
    fopen(node(idx).handle);
    node(idx).state = 'reset';
end

%% config database (used for assigning fixed names to nodes based on HWEUI)
config.hweui = {'0004A30B00E9E732','0004A30B00E9D0D2', '0004A30B00E9A493', '0004A30B00E996E9', '0004A30B00E94146', '0004A30B00E9BF72', '0004A30B00EC15E4', '0004A30B00E931E4', '0004A30B00E933E1'};
config.appeui = '0000000000000000';
config.appkey = '12345678901234567890123456789012';
      
%% operation
fprintf('----- Joining nodes... -----\n');
for idx = 1:length(node)
    n = node(idx);
    h = n.handle;
    flushinput(h);
    
    fprintf('%d/%d: resetting...', idx, numel(node));
    rn2483_cmd(h, 'sys reset', 1);
    
    fprintf('\b\b\b, get_hweui...');
    n.hweui_hex = rn2483_cmd(h, 'sys get hweui', 1);
    config_idx = find(strcmp(config.hweui, n.hweui_hex));
    n.device_name = sprintf('test_%02d', config_idx-1);
    n.devaddr = n.hweui_hex(9:16);
     
    fprintf('\b\b\b, assigned_name=''%s'', set devaddr...', n.device_name);
    rn2483_cmd(h,sprintf('mac set devaddr %s', n.devaddr), 1);
    
    fprintf('\b\b\b, set sync...');
    rn2483_cmd(h, 'mac set sync 34', 1);    
    
    fprintf('\b\b\b, set classC...');
    rn2483_cmd(h, 'mac set class c', 1);    
    
    fprintf('\b\b\b, set appeui...');
    rn2483_cmd(h, sprintf('mac set appeui %s', config.appeui), 1);    

    fprintf('\b\b\b, set appkey...');
    rn2483_cmd(h, sprintf('mac set appkey %s', config.appkey), 1);    

    for retry_cnt = 1:3
        if retry_cnt == 1
            fprintf('\b\b\b, joining...');
        else
            fprintf('\b\b\b,retry%d...', retry_cnt-1);
        end
        response = rn2483_cmd(h, 'mac join otaa', 1);
        if ~isequal(response, 'ok')
            error('unexpected response');
        end
        response = rn2483_cmd(h, [], 15);
        if isequal(response, 'accepted')
            retry_cnt = [];
            break;
        end
    end
    if ~isempty(retry_cnt)
        error('couldn''t join');
    end
    
    fprintf('\b\b\b, set dcycle...');
    for ch = 0:15
        rn2483_cmd(h, sprintf('mac set ch dcycle %d 0', ch), 1);
    end
    
    node(idx) = n;
    fprintf('\b\b\b. done.\n');
end

fprintf('Sorting node indices by device name...\n');
[~, sortidx] = sort({node.device_name}); 
node = node(sortidx);

%% Example 1
fprintf('----- Sending a single uplink message... -----\n');
for idx = 1:length(node)
    n = node(idx);
    h = n.handle;
    
    flushinput(h);
    
    for retry_cnt = 1:3
        if retry_cnt == 1
        fprintf('%d/%d (0x%s): mac tx...', idx, numel(node), n.hweui_hex);
        else
            fprintf('\b\b\b,retry%d...', retry_cnt-1);
        end
        response = rn2483_cmd(h, 'mac tx uncnf 1 313233', 2);
        if ~isequal(response, 'ok')
            error('mac tx didn''t return ''ok'' (it returned ''%s'')', response);
        end
        t = clock;
        response = rn2483_cmd(h, [], 10);
        switch (response)
            case 'busy',        continue;
            case 'mac_tx_ok',   retry_cnt = [];
                                break;
            otherwise,          error('max tx'' second reply has unexpected response');
        end
    end
    if ~isempty(retry_cnt)
        fprintf('\b\b\b. unsuccessful.\n');
        warning('mac tx unsuccessful.');
    else
        fprintf('\b\b\b. t=%.2fs, done.\n', etime(clock, t));
    end

end

%% Example 2
fprintf('----- Displaying configured frequencies... -----\n');
for idx = 1:length(node)
    n = node(idx);
    h = n.handle;
    
    fprintf('%d (0x%s):\n', idx, n.hweui_hex);
    rn2483_showfrequencies(h);
end